/*
 *    Copyright (c) 2018-2025, lengleng All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 * Redistributions of source code must retain the above copyright notice,
 * this list of conditions and the following disclaimer.
 * Redistributions in binary form must reproduce the above copyright
 * notice, this list of conditions and the following disclaimer in the
 * documentation and/or other materials provided with the distribution.
 * Neither the name of the yifu4cloud.com developer nor the names of its
 * contributors may be used to endorse or promote products derived from
 * this software without specific prior written permission.
 * Author: lengleng (wangiegie@gmail.com)
 */
package com.yifu.cloud.plus.v1.yifu.salary.service.impl;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.yifu.cloud.plus.v1.yifu.common.core.constant.CommonConstants;
import com.yifu.cloud.plus.v1.yifu.common.core.util.Common;
import com.yifu.cloud.plus.v1.yifu.common.core.util.ErrorMessage;
import com.yifu.cloud.plus.v1.yifu.common.core.util.R;
import com.yifu.cloud.plus.v1.yifu.salary.entity.TSalaryStandardOriginal;
import com.yifu.cloud.plus.v1.yifu.salary.entity.TSalaryStandardOriginalDetail;
import com.yifu.cloud.plus.v1.yifu.salary.mapper.TSalaryStandardOriginalMapper;
import com.yifu.cloud.plus.v1.yifu.salary.service.SalaryUploadService;
import com.yifu.cloud.plus.v1.yifu.salary.service.TSalaryStandardOriginalDetailService;
import com.yifu.cloud.plus.v1.yifu.salary.service.TSalaryStandardOriginalService;
import com.yifu.cloud.plus.v1.yifu.salary.util.SalaryConstants;
import com.yifu.cloud.plus.v1.yifu.salary.vo.*;
import lombok.AllArgsConstructor;
import lombok.extern.log4j.Log4j2;
import org.apache.poi.hssf.usermodel.*;
import org.apache.poi.ss.usermodel.CellType;
import org.apache.poi.xssf.usermodel.XSSFCell;
import org.apache.poi.xssf.usermodel.XSSFRow;
import org.apache.poi.xssf.usermodel.XSSFSheet;
import org.apache.poi.xssf.usermodel.XSSFWorkbook;
import org.springframework.stereotype.Service;
import org.springframework.web.multipart.MultipartFile;

import java.io.IOException;
import java.io.InputStream;
import java.math.BigDecimal;
import java.text.DecimalFormat;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

/**
 * 薪资原表识别
 *
 * @author hgw
 * @date 2024-4-2 17:20:46
 */
@Log4j2
@Service
@AllArgsConstructor
public class TSalaryStandardOriginalServiceImpl extends ServiceImpl<TSalaryStandardOriginalMapper, TSalaryStandardOriginal>
		implements TSalaryStandardOriginalService {

	private final SalaryUploadService salaryUploadService;
	private final TSalaryStandardOriginalDetailService tSalaryStandardOriginalDetailService;

	/**
	 * @Description: 分页查询
	 * @Author: hgw
	 * @Date: 2024/4/2 17:19
	 * @return:
	 **/
	@Override
	public IPage<TSalaryStandardOriginal> getTSalaryStandardOriginalPage(Page<TSalaryStandardOriginal> page, TSalaryStandardOriginal tSalaryStandardOriginal) {
		return baseMapper.getTSalaryStandardOriginalPage(page, tSalaryStandardOriginal);
	}
	/**
	 * @Description: 分页查询
	 * @Author: hgw
	 * @Date: 2024/4/2 17:19
	 * @return:
	 **/
	@Override
	public List<TSalaryStandardOriginal> getTSalaryStandardOriginalList(TSalaryStandardOriginal tSalaryStandardOriginal) {
		return baseMapper.getTSalaryStandardOriginalList(tSalaryStandardOriginal);
	}

	/**
	 * @param file
	 * @Description: 核心解析
	 * @Author: hgw
	 * @Date: 2024/4/8 17:05
	 * @return: com.yifu.cloud.plus.v1.yifu.common.core.util.R<com.yifu.cloud.plus.v1.yifu.salary.vo.ExcelVo>
	 **/
	@Override
	public R<ExcelSheetVo> getExcelHeader(MultipartFile file, String sheetName) {
		// 特殊身份证允许识别为数据行
		List<String> specialIdCardList = baseMapper.getSpecialIdCardList();
		try {
			//对前端传递的文件进行校验
			if (file == null || file.isEmpty()) {
				return R.failed("文件为空，重新上传");
			}
			//对前端传递的文件进行校验
			if (Common.isEmpty(sheetName)) {
				return R.failed("sheet名为空，请选择！");
			}
			//获取文件名称 判断文件是否为 Execl
			String filename = file.getOriginalFilename();
			if (filename == null || Common.isEmpty(filename)) {
				return R.failed("文件名称为空，请重新上传！");
			}
			if (!(filename.endsWith(".xls") || filename.endsWith(".xlsx"))) {
				return R.failed("文件名称格式有误，请重新上传！" + filename);
			}
			ExcelSheetVo vo;
			InputStream inputStream = file.getInputStream();
			//根据文件格式 对应不同的api解析
			if (filename.endsWith(".xlsx")) {
				vo = readXlsx(inputStream, specialIdCardList, true, sheetName);
			} else {
				vo = readXls(inputStream, specialIdCardList, true, sheetName);
			}
			//数据保存
			return R.ok(vo);
		} catch (IOException e) {
			e.printStackTrace();
		}
		return null;
	}

	/**
	 * @param inputStream
	 * @param specialIdCardList
	 * @param titleFlag         true:返回title;false:返回content
	 * @Description: 解析xls
	 * @Author: hgw
	 * @Date: 2024/4/8 17:02
	 * @return: com.yifu.cloud.plus.v1.yifu.salary.vo.ExcelVo
	 **/
	private ExcelSheetVo readXls(InputStream inputStream, List<String> specialIdCardList, boolean titleFlag
			, String sheetName) throws IOException {
		HSSFWorkbook sheets = null;
		ExcelSheetVo sheetVo = new ExcelSheetVo();
		try {
			List<ExcelColumnVo> titleList;
			List<ExcelColumnVo> columnList;
			ExcelColumnVo columnVo;
			// 内容list
			List<Map<String, String>> contentList;
			// 每一行的内容map
			Map<String, String> contentMap;
			sheets = new HSSFWorkbook(inputStream);
			// sheet
			HSSFSheet sheetAt;
			HSSFRow row;
			// 获取sheet数量
			int sheetNum = sheets.getNumberOfSheets();
			// 每个单元格内容
			String cellValue;
			// 内容行
			int dataRow;
			// 数据行是否被确认，true否
			boolean dataRowFlag;
			HSSFCell cell;
			// 循环每个sheet
			for (int i = 0; i < sheetNum; i++) {
				sheetAt = sheets.getSheetAt(i);
				if (Common.isNotNull(sheetAt.getSheetName()) && Common.isNotNull(sheetName)
						&& sheetName.equals(sheetAt.getSheetName())) {
					contentList = new ArrayList<>();
					// 塞sheet名称
					sheetVo.setSheetName(sheetAt.getSheetName());
					titleList = new ArrayList<>();
					// 初始化每个sheet的数据行
					dataRow = 0;
					dataRowFlag = true;
					// 循环当前sheet的每行内容
					for (int rowNum = 0; rowNum < sheetAt.getLastRowNum(); rowNum++) {
						contentMap = new HashMap<>();
						columnList = new ArrayList<>();
						row = sheetAt.getRow(rowNum);
						if (row != null) {
							for (int cellNum = 0; cellNum < row.getLastCellNum(); cellNum++) {
								columnVo = new ExcelColumnVo();
								try {
									cell = row.getCell(cellNum);
									if (CellType.STRING == cell.getCellType()) {
										cellValue = row.getCell(cellNum).getStringCellValue();
									} else if (CellType.NUMERIC == cell.getCellType()) {
										try {
											cellValue = String.valueOf(row.getCell(cellNum).getNumericCellValue());
											if (cellValue.contains("E")) {
												cellValue = String.valueOf(new DecimalFormat("#").format(row.getCell(cellNum).getNumericCellValue()));
											}
										} catch (NumberFormatException e) {
											cellValue = String.valueOf(row.getCell(cellNum).getNumericCellValue());
										}
									} else if (HSSFDateUtil.isCellDateFormatted(cell)) {
										cellValue = String.valueOf(row.getCell(cellNum).getDateCellValue());
									} else if (CellType.BOOLEAN == cell.getCellType()) {
										cellValue = String.valueOf(row.getCell(cellNum).getBooleanCellValue());
									} else {
										cellValue = row.getCell(cellNum).getStringCellValue();
									}
								} catch (Exception e) {
									sheetVo.setErrorInfo("请注意单元格格式，第"+(i+1) +"个sheet，第"+(rowNum+1)+"行第"+(cellNum+1)+"列。"+e.getMessage());
									return sheetVo;
								}
								if (Common.isNotNull(cellValue)) {
									if (dataRowFlag
											&& (((cellValue.length() == 15 || cellValue.length() == 18) && regIdCard(cellValue))
											// 特殊身份证允许识别
											|| regSpecialIdCard(cellValue, specialIdCardList))) {
										dataRow = rowNum;
										dataRowFlag = false;
									}
									columnVo.setLineNum(cellNum);
									columnVo.setContent(cellValue);
									contentMap.put(String.valueOf(cellNum), cellValue);
									if (dataRowFlag) {
										columnList.add(columnVo);
									}
								}
							}
							if (!dataRowFlag) {
								contentList.add(contentMap);
							} else {
								titleList = new ArrayList<>(columnList);
							}
						}
						if (titleFlag && !dataRowFlag) {
							break;
						}
					}
					sheetVo.setDataNum(dataRow);
					sheetVo.setTitleList(titleList);
					sheetVo.setContentList(contentList);
					return sheetVo;
				}
			}
		} catch (IOException e) {
			e.printStackTrace();
			if (sheets != null) {
				sheets.close();
			}
		} finally {
			if (sheets != null) {
				sheets.close();
			}
		}
		return sheetVo;
	}

	/**
	 * @param inputStream
	 * @param specialIdCardList
	 * @param titleFlag         true:返回title;false:返回content
	 * @Description: 解析xlsx
	 * @Author: hgw
	 * @Date: 2024/4/8 17:02
	 * @return: com.yifu.cloud.plus.v1.yifu.salary.vo.ExcelVo
	 **/
	private ExcelSheetVo readXlsx(InputStream inputStream, List<String> specialIdCardList, boolean titleFlag
			, String sheetName) throws IOException {
		XSSFWorkbook sheets = null;
		ExcelSheetVo sheetVo = new ExcelSheetVo();
		try {
			List<ExcelColumnVo> titleList;
			List<ExcelColumnVo> columnList;
			ExcelColumnVo columnVo;
			// 内容list
			List<Map<String, String>> contentList;
			// 每一行的内容map
			Map<String, String> contentMap;
			sheets = new XSSFWorkbook(inputStream);
			// sheet
			XSSFSheet sheetAt;
			XSSFRow row;
			// 获取sheet数量
			int sheetNum = sheets.getNumberOfSheets();
			// 每个单元格内容
			String cellValue;
			// 内容行
			int dataRow;
			// 数据行是否被确认，true否
			boolean dataRowFlag;
			XSSFCell cell;
			// 循环每个sheet
			for (int i = 0; i < sheetNum; i++) {
				sheetAt = sheets.getSheetAt(i);
				if (Common.isNotNull(sheetAt.getSheetName()) && Common.isNotNull(sheetName)
						&& sheetName.equals(sheetAt.getSheetName())) {
					contentList = new ArrayList<>();
					// 塞sheet名称
					sheetVo.setSheetName(sheetAt.getSheetName());
					titleList = new ArrayList<>();
					// 初始化每个sheet的数据行
					dataRow = 0;
					dataRowFlag = true;
					// 循环当前sheet的每行内容
					for (int rowNum = 0; rowNum < sheetAt.getLastRowNum(); rowNum++) {
						contentMap = new HashMap<>();
						columnList = new ArrayList<>();
						row = sheetAt.getRow(rowNum);
						if (row != null) {
							for (int cellNum = 0; cellNum < row.getLastCellNum(); cellNum++) {
								columnVo = new ExcelColumnVo();
								try {
									cell = row.getCell(cellNum);
									if (CellType.STRING == cell.getCellType()) {
										cellValue = row.getCell(cellNum).getStringCellValue();
									} else if (CellType.NUMERIC == cell.getCellType()) {
										try {
											cellValue = String.valueOf(row.getCell(cellNum).getNumericCellValue());
											if (cellValue.contains("E")) {
												cellValue = String.valueOf(new DecimalFormat("#").format(row.getCell(cellNum).getNumericCellValue()));
											}
										} catch (NumberFormatException e) {
											cellValue = String.valueOf(row.getCell(cellNum).getNumericCellValue());
										}
									} else if (HSSFDateUtil.isCellDateFormatted(cell)) {
										cellValue = String.valueOf(row.getCell(cellNum).getDateCellValue());
									} else if (CellType.BOOLEAN == cell.getCellType()) {
										cellValue = String.valueOf(row.getCell(cellNum).getBooleanCellValue());
									} else {
										cellValue = row.getCell(cellNum).getStringCellValue();
									}
								} catch (Exception e) {
									sheetVo.setErrorInfo("请注意单元格格式，第"+(i+1) +"个sheet，第"+(rowNum+1)+"行第"+(cellNum+1)+"列。"+e.getMessage());
									return sheetVo;
								}
								if (Common.isNotNull(cellValue)) {
									if (dataRowFlag
											&& (((cellValue.length() == 15 || cellValue.length() == 18) && regIdCard(cellValue))
											// 特殊身份证允许识别
											|| regSpecialIdCard(cellValue, specialIdCardList))) {
										dataRow = rowNum;
										dataRowFlag = false;
									}
									columnVo.setLineNum(cellNum);
									columnVo.setContent(cellValue);
									contentMap.put(String.valueOf(cellNum), cellValue);
									if (dataRowFlag) {
										columnList.add(columnVo);
									}
								}
							}
							if (!dataRowFlag) {
								contentList.add(contentMap);
							} else {
								titleList = new ArrayList<>(columnList);
							}
						}
						if (titleFlag && !dataRowFlag) {
							break;
						}
					}
					sheetVo.setDataNum(dataRow);
					sheetVo.setTitleList(titleList);
					sheetVo.setContentList(contentList);
					return sheetVo;
				}
			}
		} catch (IOException e) {
			e.printStackTrace();
			if (sheets != null) {
				sheets.close();
			}
		} finally {
			if (sheets != null) {
				sheets.close();
			}
		}
		return sheetVo;
	}

	/**
	 * @param content
	 * @Description: 判断特殊身份证
	 * 格式正确返回true
	 * @Author: hgw
	 * @Date: 2024-4-8 15:33:04
	 * @return: boolean
	 **/
	private static boolean regSpecialIdCard(String content, List<String> specialIdCardList) {
		if (specialIdCardList != null && !specialIdCardList.isEmpty()) {
			for (int i = 0; i < specialIdCardList.size(); i++) {
				if (specialIdCardList.get(i).equals(content)) {
					return true;
				}
			}
		}
		return false;
	}

	/**
	 * @param content
	 * @Description: 判断身份证格式：18位数字，17位数字+X，15位数字，14位数字+X
	 * 格式正确返回true
	 * @Author: hgw
	 * @Date: 2022/5/13 11:53
	 * @return: boolean
	 **/
	private static boolean regIdCard(String content) {
		boolean flag = false;
		Pattern p = Pattern.compile("\\d{18}|\\d{17}X|\\d{15}|\\d{14}X");
		Matcher m = p.matcher(content);
		if (m.matches()) {
			flag = true;
		}
		return flag;
	}

	/**
	 * @param excelVo
	 * @Description: 新版原表识别导入
	 * @Author: hgw
	 * @Date: 2024/4/8 18:22
	 * @return: com.yifu.cloud.plus.v1.yifu.common.core.util.R<java.util.List < com.yifu.cloud.plus.v1.yifu.common.core.util.ErrorMessage>>
	 **/
	@Override
	public R<List<ErrorMessage>> salaryUploadByOriginal(SalaryUploadOriginalParamVo excelVo) {
		R<String> contR = this.getExcelContent(excelVo);
		if (contR != null) {
			if (CommonConstants.SUCCESS.equals(contR.getCode())) {
				SalaryUploadParamVo vo = new SalaryUploadParamVo();
				vo.setJsonString(contR.getData());
				vo.setSalaryType(excelVo.getSalaryType());
				vo.setConfigId(excelVo.getConfigId());
				vo.setIsIssue(excelVo.getIsIssue());
				vo.setOrderId(excelVo.getOrderId());
				vo.setResList(excelVo.getResList());
				vo.setSettleDepart(excelVo.getSettleDepart());
				return salaryUploadService.salaryUpload(vo);
			} else {
				return R.failed(contR.getMsg());
			}
		} else {
			return R.failed("原表识别无数据！请准备原表，联系管理员");
		}
	}

	/**
	 * @param excelVo
	 * @Description: 核心解析-内容
	 * @Author: hgw
	 * @Date: 2024-4-8 18:20:22
	 * @return: com.yifu.cloud.plus.v1.yifu.common.core.util.R<com.yifu.cloud.plus.v1.yifu.salary.vo.ExcelVo>
	 **/
	private R<String> getExcelContent(SalaryUploadOriginalParamVo excelVo) {
		MultipartFile file = excelVo.getFile();
		String sheetName = excelVo.getSheetName();
		// 配置
		TSalaryStandardOriginal original = excelVo.getOriginal();
		// 特殊身份证允许识别为数据行
		List<String> specialIdCardList = baseMapper.getSpecialIdCardList();
		try {
			//对前端传递的文件进行校验
			if (original == null || Common.isEmpty(original.getId())) {
				return R.failed("识别配置为空，请选择识别配置！");
			}
			List<TSalaryStandardOriginalDetail> detailList = tSalaryStandardOriginalDetailService.getTSalaryStandardOriginalDetailByOrgId(original.getId());
			Map<String, TSalaryStandardOriginalDetail> detailMap = new HashMap<>();
			for (TSalaryStandardOriginalDetail detail : detailList) {
				detailMap.put(String.valueOf(detail.getColumnNo()), detail);
			}
			//对前端传递的文件进行校验
			if (Common.isEmpty(sheetName)) {
				return R.failed("sheet名为空，请选择！");
			}
			//对前端传递的文件进行校验
			if (file == null || file.isEmpty()) {
				return R.failed("文件为空，重新上传");
			}
			//获取文件名称 判断文件是否为 Execl
			String filename = file.getOriginalFilename();
			if (filename == null || Common.isEmpty(filename)) {
				return R.failed("文件名称为空，请重新上传！");
			}
			if (!(filename.endsWith(".xls") || filename.endsWith(".xlsx"))) {
				return R.failed("文件名称格式有误，请重新上传！" + filename);
			}
			String jsonString = null;
			InputStream inputStream = file.getInputStream();
			ExcelSheetVo vo;
			//根据文件格式 对应不同的api解析
			if (filename.endsWith(".xlsx")) {
				vo = readXlsx(inputStream, specialIdCardList, false, sheetName);
			} else {
				vo = readXls(inputStream, specialIdCardList, false, sheetName);
			}
			if (vo.getContentList() != null && !vo.getContentList().isEmpty()) {
				// 智能识别表头
				TSalaryStandardOriginalDetail detail;
				List<Map<String, String>> contentList = new ArrayList<>();
				Map<String, String> newRowMap;
				String value;
				String oldValue;
				int i=0;
				for (Map<String, String> rowMap : vo.getContentList()) {
					i++;
					newRowMap = new HashMap<>();
					for (Map.Entry<String, String> column : rowMap.entrySet()) {
						detail = detailMap.get(column.getKey());
						if (detail != null) {
							value = column.getValue();
							try {
								if ("REDUCE".equals(detail.getFlag())) {
									value = String.valueOf(new BigDecimal(value).negate());
								}
								oldValue = newRowMap.get(detail.getModelName());
								if (oldValue != null) {
									value = String.valueOf(new BigDecimal(oldValue).add(new BigDecimal(value))
											.setScale(SalaryConstants.PLACES, BigDecimal.ROUND_HALF_UP));
								}
							} catch (NumberFormatException e) {
								return R.failed("第 "+ i + " 行，第 "+ (Integer.parseInt(column.getKey())+1) +" 列 不是金额，不可以进行加减计算！" );
							}
							newRowMap.put(detail.getModelName(), value);
						}
					}
					contentList.add(newRowMap);
				}
				// 最后转化为jsonString
				jsonString = JSON.toJSON(contentList).toString();
			}
			//数据保存
			return R.ok(jsonString);
		} catch (IOException e) {
			e.printStackTrace();
		}
		return null;
	}

}
